local const = require("const")
local json = require("json")
local utils = require("utils")
local utils_device = require("utils_device")
local resty_string = require("resty.string")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DeviceManage")

if ("POST" == request_method) then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    if (nil == args_json) then
        utils.error_exit(400, json.encode("request is invalid"))
    end

    local args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    local request = args["request"]
    if (request == nil) then
        utils.error_exit(400, json.encode("request is invalid"))
    end

    local apiVer = request["apiVer"] 
    if (apiVer ~= "0.6") then
        utils.error_exit(400, json.encode("apiVer is invalid"))
    end
    
    local params = args["params"]
    if(nil == params) then
        utils.error_exit(400, json.encode("params should not be null"))
    end
   
    local pk = params["productKey"]
    local dn = params["deviceName"]

    if (not (dn and pk)) then
        utils.error_exit(400, json.encode("productKey or deviceName is invalid."))
    end
    
    utils.check_pk(pk); 
    utils.check_dn(dn); 

    local ret, ret_msg = utils.sync_device_tags()
    if (ret == true) then
        utils.error_exit(200, json.encode(ret_msg))
    else
        utils.error_exit(500, json.encode(ret_msg))
    end
end

utils.error_exit(400, "Invalid request method")
