
local json = require("json")
local utils = require("utils")
local driver_utils = require("driver_utils")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("DriverManage") 

if ("POST" == request_method) then 
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    ok, args = pcall(json.decode, args_json)
    if (not ok or nil == args) then 
        utils.error_exit(400, args)
    end

    local driver_name = args["DriverName"]
    local config = args["Config"]
    if (not (driver_name and config and "table" == type(config))) then
        utils.error_exit(400)
    end
    utils.set_driver_config(driver_name, json.encode(config))
    -- modified by tiger.
    -- driver_utils.set_driver_conf(driver_name, config)
    utils.error_exit(200)
    
elseif ("GET" == request_method) then 
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    driver_name = args["DriverName"]
    if (not driver_name) then
        utils.error_exit(400, "invalid parameter")
    end

    msg = utils.get_driver_config(driver_name)
    -- msg = driver_utils.get_driver_conf(driver_name)
    res = {}
    res["DriverName"] = driver_name
    res["Config"] = json.decode(msg)
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)
end

utils.error_exit(400, "unsupported request method")
------ EOF ------
