#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging
from . import sync
from . import config
from . import ws_client
from . import fc_error
from . import utils


class IPC_Wrapper(object):
    _instance = None

    def __init__(self, functionId, auth):
        self.ws = ws_client.get_ws_client()
        self.functionId = functionId
        self.auth = auth
        self.getParamSyncDict = {}
        self.postResultSyncDict = {}
        self.postHandlerErrSyncDict = {}
        self.invokeTaskSyncDict = {}
        self.getTaskResultSyncDict = {}
        self.getCredentialSyncDict = {}

    def __del__(self):
        self.ws.close()

    def __new__(self, *args, **kw):
        if not self._instance:
            self._instance = super(IPC_Wrapper, self).__new__(self)
        return self._instance

    def _getParamResult_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            getParamSync = self.getParamSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            getParamSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def get_param(self):
        options = {"method": 'GET',
                   "path": utils.getTaskRequestPath(self.functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth}
                   }
        getParamSync = sync.SyncMsg_Event()
        getParamSync.clear()
        reply_cb = self._getParamResult_cb
        uuid = utils.get_uuid()
        self.getParamSyncDict[uuid] = getParamSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = getParamSync.wait()
        self.getParamSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="get param timeout")
            return None
        logging.debug("get param:%s", syncMsg)
        return syncMsg

    def _postTaskResult_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            postResultSync = self.postResultSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            postResultSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def postTaskResult(self, invocationId, result):
        options = {"method": 'POST',
                   "path": utils.getTaskRequestPath(self.functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth,
                       config.HEADER_INVOCATION_ID: invocationId},
                   "body": result
                   }
        postResultSync = sync.SyncMsg_Event()
        postResultSync.clear()
        reply_cb = self._postTaskResult_cb
        uuid = utils.get_uuid()
        self.postResultSyncDict[uuid] = postResultSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = postResultSync.wait(config.MERHOD_SYNC_TIMEOUT)
        self.postResultSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("postTaskResult:%s", syncMsg)
        return syncMsg

    def _postHandlerErr_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            postHandlerErrSync = self.postHandlerErrSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            postHandlerErrSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def postHandlerErr(self, invocationId, error):
        options = {"method": 'POST',
                   "path": utils.getTaskRequestPath(self.functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth,
                       config.HEADER_INVOCATION_ID: invocationId,
                       config.HEADER_FUNCTION_ERROR: 'Handled'},
                   "body": error}
        postHandlerErrSync = sync.SyncMsg_Event()
        postHandlerErrSync.clear()
        reply_cb = self._postTaskResult_cb
        uuid = utils.get_uuid()
        self.postHandlerErrSyncDict[uuid] = postHandlerErrSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = postHandlerErrSync.wait(config.MERHOD_SYNC_TIMEOUT)
        self.postHandlerErrSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("postHandlerErr:%s", syncMsg)
        return syncMsg

    def _invokeTask_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            invokeTaskSync = self.invokeTaskSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            invokeTaskSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def invokeTask(self, functionId, invokerContext, invocationType, payload):
        options = {"method": 'POST',
                   "path": utils.getRequestPath(functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth,
                       config.HEADER_INVOKER_CONTEXT: invokerContext,
                       config.HEADER_INVOCATION_TYPE: invocationType},
                   "body": payload}
        invokeTaskSync = sync.SyncMsg_Event()
        invokeTaskSync.clear()
        reply_cb = self._invokeTask_cb
        uuid = utils.get_uuid()
        self.invokeTaskSyncDict[uuid] = invokeTaskSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = invokeTaskSync.wait(config.MERHOD_SYNC_TIMEOUT)
        self.invokeTaskSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="invokeTask timeout")
            return None
        logging.debug("invokeTask:%s", syncMsg)
        return syncMsg

    def _getTaskResult_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            getTaskResultSync = self.getTaskResultSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            getTaskResultSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def getTaskResult(self, functionId, invocationId):
        options = {"method": 'GET',
                   "path": utils.getRequestPath(functionId),
                   "headers": {
                       config.HEADER_AUTH_TOKEN: self.auth,
                       config.HEADER_INVOCATION_ID: invocationId}
                   }
        getTaskResultSync = sync.SyncMsg_Event()
        getTaskResultSync.clear()
        reply_cb = self._getTaskResult_cb
        uuid = utils.get_uuid()
        self.getTaskResultSyncDict[uuid] = getTaskResultSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = getTaskResultSync.wait()
        self.getTaskResultSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="post task result timeout")
            return None
        logging.debug("getTaskResult:%s", syncMsg)
        return syncMsg

    def _getCredential_cb(self, uuid, status, data):
        syncMsg = {}
        syncMsg["state"] = False
        syncMsg["msg"] = None

        try:
            getCredentialSync = self.getCredentialSyncDict[uuid]
            logging.debug("staus:%s", status)
            if status == "success":
                syncMsg["state"] = True
                syncMsg["msg"] = data
            else:
                syncMsg["msg"] = ''
                syncMsg["state"] = False
            getCredentialSync.set(syncMsg)
        except Exception as e:
            logging.error("no uuid %s. %s", uuid, e)

    def get_credential(self, uri):
        options = {
            "method": 'GET',
            "path": uri,
            "headers": {config.HEADER_AUTH_TOKEN: self.auth}}
        getCredentialSync = sync.SyncMsg_Event()
        getCredentialSync.clear()
        reply_cb = self._getCredential_cb
        uuid = utils.get_uuid()
        self.getCredentialSyncDict[uuid] = getCredentialSync
        self.ws.request(uuid, options, reply_cb)
        syncMsg = getCredentialSync.wait()
        self.getCredentialSyncDict.pop(uuid)
        if (None == syncMsg):
            raise fc_error.WSConnException(msg="get param timeout")
            return None
        logging.debug("get_credential:%s", syncMsg)
        return syncMsg


def get_ipc_wrapper():
    return IPC_Wrapper(os.environ.get("FUNCTION_ID"), os.environ.get("AUTH_TOKEN"))


ipc = get_ipc_wrapper()
