'use strict'

const config = require('./config.js');

var dbus;
if (config.RUN_MODE === config.MODE_OPTION.REAL) {
    dbus = require("dbus_wrap.node");
} else {
    dbus = require("./MockDBus.js");
}
/**
 * DBus proxy, which wrap a dbus server.
 * @param {[String]} wellknowName [the dbus server name]
 * @param {[String]} objPath      [the dbus server object path]
 * @param {[String]} interface    [the dbus server interface]
 */
var DBusProxy = function(wellknowName, objPath, iface) {
    this.wellknowName = wellknowName;
    this.objPath = objPath;
    this.interface = iface;
    // this.proxy = new dbus.DBusProxy(wellknowName, objPath, iface);
};

const ProxyWellKnowName = "iot.runtime.js.proxy";
const ProxyObjPath = "/iot/runtime/js/proxy";
const ProxyIface = "iot.runtime.js.proxy";
DBusProxy.proxy = new dbus.DBusProxy(ProxyWellKnowName, ProxyObjPath, ProxyIface);

DBusProxy.isServiceOnline = function(wellknowName) {
    return DBusProxy.proxy.isServiceOnline(wellknowName);
};
/**
 * send a signal to the wrapped server
 * @param  {String}    signal [signal name]
 * @param  {...[type]} args   [arguments list, now support Number/String/Boolean]
 * @return {[None]}
 */
DBusProxy.prototype.sendSignal = function(signal, ...args) {
    try {
        DBusProxy.proxy.sendSignal(this.wellknowName, this.objPath, this.interface, signal, ...args);
    } catch(e) {
        // console.log(e);
    }
};

DBusProxy.prototype.sendTargetSignal = function (wellknownName, objPath, _interface, signal, ...args) {
    try {
        DBusProxy.proxy.sendTargetSignal(wellknownName, objPath, _interface, signal, ...args);
    } catch(e) {
        // console.log(e);
    }
};

/**
 * call a remote method async to the wrapped server
 * @param  {String}    method [method name]
 * @param  {...[type]} args   [arguments list, now support Number/String/Boolean, the last argument
 *                            must be callback function, who's prototype is:
 *                            function(err, result) {}]
 * @return {[None]}
 */
DBusProxy.prototype.callServiceAsync = function(method, ...args) {
    try {
        return DBusProxy.proxy.callServiceAsync(this.wellknowName, this.objPath, this.interface, method, ...args);
    } catch(e) {
        // console.log(e);
    }
};

/**
 * call a remote method sync to the wrapped server
 * @param  {String}    method [method name]
 * @param  {...[type]} args   [arguments list, now support Number/String/Boolean]
 * @return {[result]}       return value according the server.
 */
DBusProxy.prototype.callService = function(method, ...args) {
    try {
        return DBusProxy.proxy.callService(this.wellknowName, this.objPath, this.interface, method, ...args);
    } catch(e) {
         // console.log(e);
    }
};


DBusProxy.prototype.getConfig = function (key) {
    var array = DBusProxy.proxy.getConfig(key);
    // return (array!= null && array.length > 0) ? array[0] : "";
    return array;
};

DBusProxy.prototype.setConfig = function (key, value) {
    return DBusProxy.proxy.setConfig(key, value);
};


module.exports = DBusProxy;
