'use strict';
const config = require('./config.js');

var dbus;
if (config.RUN_MODE <= config.MODE_OPTION.MOCK_PROXY_REAL_SERVER_WITH_ALL_DMSG) {
  dbus = require("dbus_wrap.node");
} else {
  dbus = require("./MockDBus.js");
}
/**
 * DBus Server wrap, which run as server
 * @param {[String]} wellknowName [the dbus server name]
 * @param {[String]} objPath      [the dbus server object path]
 * @param {[String]} interface    [the dbus server interface]
 */
var DBusServer = function(wellknowName, objPath, iface) {
    this.wellknowName = wellknowName;
    this.objPath = objPath;
    this.interface = iface;
    this.server = new dbus.DBusServer(this.wellknowName, this.objPath);
};
/**
 * prepare dbus connecton and exe environment.
 * @return {bool} if everything is Ok, return true, otherwise false.
 */
DBusServer.prototype.prepare = function() {
    return this.server.prepare();
};
/**
 * start the dbus server
 * @return {[None]} [if start server failed, throw exception.]
 */
DBusServer.prototype.start = function() {
    this.server.start();
};

/**
 * stop the dbus server
 * @return {[none]}
 */
DBusServer.prototype.stop = function() {
    this.server.stop();
};
/**
 * add dbus match rule, which must be according with the dbus match rule specification:
 *
 * https://dbus.freedesktop.org/doc/dbus-specification.html#message-bus-routing-match-rules
 *
 * @param {[none]} rule []
 * @example
 * "type='signal',path='/iot/device/id1024',interface='iot.device.id1024'"
 * "type='method_call',path='/iot/device/id1024',interface='iot.device.id1024'"
 */
DBusServer.prototype.addMatch = function(rule) {
    this.server.addMatch(rule);
};

DBusServer.prototype.removeMatch = function(rule) {
    this.server.removeMatch(rule);
};

/**
 * dbus event subscribe
 * @param  {String}   event    [event name, 'signal' or 'method_call']
 * @param  {Function} callback [callback function, the function's prototype is
 *                             function(name, args) {}
 * ]
 * @return {None}
 */
DBusServer.prototype.on = function(event, callback) {
    this.server.on(event, callback);
};

DBusServer.prototype.sendSignal = function(signal, args) {
    return this.server.sendSignal(signal, args);
};

DBusServer.prototype.traceLog = function (level, tag, file, func, line, log) {
    this.server.traceLog(level, tag, file, func, line, log);
};

module.exports = DBusServer;
