/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reversed.
 */

var Log = require('./utils/log');
var scheduler = require("node-schedule");

var TimerTrigger = function (params, sid, eventBus) {
  this.params = params;
  this.fullfilled = false; // this.fullfilled always set to false, so onPropertiesChanged will never triggered it.
  this.trigger = function (info) {};
  this.sceneid = sid;
  this.job = undefined;
  this.eventBus = eventBus;
};

TimerTrigger.prototype.start = function () {
  var self = this;
  var cron = this.params['cron'];
  if (!cron || cron === '') {
    Log.w('Field "cron" in %s is empty, discard.', this.params);
    return undefined;
  }

  this.job = scheduler.scheduleJob(cron, function () {
    Log.i('Scheduled job event: {%s, %s} trigger...', self.sceneid, cron);
    self.eventBus.publish("timerEvent", {
      sceneid: self.sceneid
    });
  });

  if (!this.job) {
    Log.w('Schedule job {%s, %s} failed.', self.sceneid, cron);
  }
  return this.job;
};

TimerTrigger.prototype.stop = function () {
  if (this.job) {
    scheduler.cancelJob(this.job);
  }
  return true;
};

TimerTrigger.prototype.eventHappen = function () {
  return false;
};

module.exports = TimerTrigger;
